/*
Alle Funktionen ohne Gewhr
Michael Nowak
2019-03-20
www.mino-elektronik.de
*/

#define _TOUCH

#include <stdint.h>
#include <string.h>
#include "touch.h"

#include "stm32f7xx.h"
#include "dma_tft.h"
#include "dma_tft.ext"

// typische ADC-Werte fr 4,3" Touch-Folien
#define X1              70               // oben links
#define Y1              150
#define X2              940              // unten rechts
#define Y2              900
#define ENTPRELLZEIT    5
#define NACHLAUFZEIT    (ENTPRELLZEIT*2)


static volatile int32_t touch_x=0, touch_y=0;   // Ergebnis der Dekodierung
static volatile int32_t temp_x=0, temp_y=0;
static volatile int32_t x_wert=0, y_wert=0;
static volatile int32_t touch_status;           // Information gedrueckt + repeat

volatile uint8_t sync_flag;

static volatile uint32_t timer = 0;
static volatile short tft_alarm_count, t_blink = 350;
volatile uint8_t blink_bit=0;
volatile uint8_t touch_aktiv;
	

// Zuordnung der Touch-Leitungen an PortA
/*
#define TPXL GPIO_PIN_0
#define TPYU GPIO_PIN_1
#define TPXR GPIO_PIN_2
#define TPYD GPIO_PIN_3
*/
#define XL 0
#define YU 1
#define XR 2
#define YD 3

static inline void x_achse_passiv(void)                // TPXL = 0, TPXR passiv
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (XL * 2) |  // XL loeschen
                         GPIO_MODER_MODER0 << (XR * 2));  // XR loeschen
}

static inline void x_achse_0(void)            // TPXR = 1, TPXL = 0
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (XL * 2) |  // XL loeschen
                         GPIO_MODER_MODER0 << (XR * 2));  // XR loeschen
      GPIOA->MODER |=  1 << (XR * 2);                     // XR auf Ausgang
      set_pin(GPIOA, XR, 0);	                      // XR auf 0
}
static inline void x_achse_aktiv(void)            // TPXR = 1, TPXL = 0
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (XL * 2) |  // XL loeschen
                         GPIO_MODER_MODER0 << (XR * 2));  // XR loeschen
      GPIOA->MODER |= (1 << (XL * 2) |                    // XL auf Ausgang
                       1 << (XR * 2));                    // XR auf Ausgang
      set_pin(GPIOA, XR, 1);	                      // XR auf V+
}

static inline void x_achse_adc(void)
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (XL * 2) |  // XL loeschen
                         GPIO_MODER_MODER0 << (XR * 2));  // XR loeschen
      GPIOA->MODER  |= 3 << (XL * 2);  // auf analogen Eingang setzen
}

static inline void y_achse_high(void)               // TPYD = 1, TPYU = passiv 
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (YD * 2) |  // YD loeschen
                         GPIO_MODER_MODER0 << (YU * 2));  // YU loeschen
      set_pin(GPIOA, YD, 1);	                      // YD auf V+
      GPIOA->MODER |=  1 << (YD * 2);                     // YD auf Ausgang
}

static inline void pullup_YD_ein(void)
{
      GPIOA->PUPDR |=  (1 << (YD * 2));     // Pullup einschalten
}

static inline void pullup_YD_aus(void)
{
      GPIOA->PUPDR &=  ~(1 << (YD * 2));    // Pullup wieder abschalten
}

static inline void y_achse_aktiv(void)            // TPYD = 1, TPYU = 0
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (YD * 2) |  // YD loeschen
                         GPIO_MODER_MODER0 << (YU * 2));  // YU loeschen
      GPIOA->MODER |= (1 << (YD * 2) |                    // YD auf Ausgang
                       1 << (YU * 2));                    // YU auf Ausgang
      set_pin(GPIOA, YD, 1);	                      // YD auf V+
}

static inline void y_achse_adc(void)           // TPYD + TPYU passiv
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (YU * 2) |  // YU loeschen
                         GPIO_MODER_MODER0 << (YD * 2));  // YD loeschen
      GPIOA->MODER  |= 3 << (YU * 2);  // auf analogen Eingang setzen
}
/*
static inline void y_achse_passiv(void)           // TPYD + TPYU passiv
{
      GPIOA->MODER  &= ~(GPIO_MODER_MODER0 << (YU * 2) |  // YU loeschen
                         GPIO_MODER_MODER0 << (YD * 2));  // YD loeschen
}
*/	


static inline int32_t lese_adc_x(void)
{
    return(ADC1->DR);
}	

static inline int32_t lese_adc_y(void)
{
    return(ADC2->DR);
}

void dekodiere_touch(void)
{
static int aktiv = 0, gedrueckt = 0, zuletzt = 0, entprellung = 0;	
static int phase, nachlauf;

	phase++;				// immer erhhen
	switch(phase) {
		case 1:				// x auf schwebend und y auf V+ schalten
            x_achse_passiv();
			y_achse_high();
			break;
		case 2:				// y auf ADC und sehen, wo der Pegel bleibt
            x_achse_0();
            pullup_YD_ein();  // y-achse auf '1' halten
			y_achse_adc();  
		break;
		case 3:				// y-wert messen
			temp_y = lese_adc_y();
			if(temp_y < 100) {      // touch ist gedrueckt
              aktiv = 1;
              nachlauf = 0;
			} else {
              aktiv = 0;
             nachlauf++;
              if(nachlauf >= NACHLAUFZEIT) {
                nachlauf--;
                phase--;      // auf Kontakt vom touch warten
              } 
              
            }  
//            phase = 0;
			break;
		case 4:				// x_achse einschalten
            pullup_YD_aus();  // pullup wegnehmen
			y_achse_adc();
			x_achse_aktiv();
			break;
		case 5:				// y_achse einlesen
			x_wert = temp_x = lese_adc_y();
			x_achse_adc();
			y_achse_aktiv();
			break;
		case 6:				// x-achse einlesen
			y_wert = temp_y = lese_adc_x();
			phase = 0;      // wieder bei case1 beginnen
			break;
		default:
			phase = 0;		// fr ungltige Zustnde	
	}	

	if(phase == 0) {			
		if(aktiv != zuletzt) {
			entprellung = 0;
			zuletzt = aktiv;
		}			
		entprellung++;
		if(entprellung > ENTPRELLZEIT) {
			entprellung--;
			if(aktiv ^ gedrueckt) {		    // nderung passiert
				gedrueckt = aktiv;
				if(gedrueckt) {             // skalieren
					touch_x = ((temp_x-X1)*SIZE_X)/ (X2-X1);
					touch_y = ((temp_y-Y1)*SIZE_Y)/ (Y2-Y1);
					if(touch_x > MAX_X) touch_x = MAX_X;
					if(touch_x < 0) touch_x = 0;
					if(touch_y > MAX_Y) touch_y = MAX_Y;
					if(touch_y < 0) touch_y = 0;
#ifdef EDT_TFT
                    touch_x = SIZE_X-touch_x;            // vertauschte Leitungen
#endif                    
					touch_aktiv = TOUCH_AKTIV;        // touch ist gedrckt
				}
				else {
					touch_aktiv = TOUCH_PASSIV;        // touch wurde losgelassen
				}
			}
		}	
	}
}

void teste_touch_status(void)                        // 
{
static int change,last_change;
static int touch_aktiv_alt,rep_counter=0, rep_flag=0;

  if(rep_counter) rep_counter--;

  change = (touch_aktiv_alt ^ touch_aktiv);
  touch_aktiv_alt = touch_aktiv;
  if(change) {
    last_change = change;
    rep_counter = FIRST_REPEAT;
    rep_flag = 0;
  } else if(rep_counter == 0 && rep_flag) {
    change = last_change & touch_aktiv;
    rep_counter = NEXT_REPEAT;
  }
  if(change) {
    if(((change & TOUCH_AKTIV) & touch_aktiv) == TOUCH_AKTIV) {
      if(rep_flag) {
        touch_status = REPEAT_TASTE + TOUCH_GEDRUECKT;
      } else {
        touch_status = NEUE_TASTE + TOUCH_GEDRUECKT;
      }
      rep_flag = 1;
    } else {
      rep_flag = 0;
    }
  }
}

/* 
PA0: touch_X = ADC123_IN0
PA1: touch_Y = ADC123_IN1
*/ 
static void adc12_init(void)
{
// ADC1 und ADC2 einschalten
  RCC->APB2ENR |= (RCC_APB2ENR_ADC1EN | RCC_APB2ENR_ADC2EN);
 
// Configure ADC Channel 0 und 1 pin als analog input
    GPIOA->MODER |= (3 << (XL * 2) |                    // XL als ADC-Eingang
                     3 << (YU * 2));                    // YU als ADC-Eingang
 
    ADC1->CR1 = ADC_CR1_RES_0;                  // 10 Bit ADC1-IN0
    ADC2->CR1 = ADC_CR1_RES_0 | ADC_CR1_AWDCH_0;// 10 Bit ADC2-IN1
    ADC1->CR2 = ADC_CR2_ADON | ADC_CR2_CONT;    // kont. Messungen
    ADC2->CR2 = ADC_CR2_ADON | ADC_CR2_CONT;    // kont. Messungen
    ADC2->SQR3 = 1; 
    ADC1->CR2 |= ADC_CR2_SWSTART;
    ADC2->CR2 |= ADC_CR2_SWSTART;

}//end adc12_init



void init_t7_touch(void)
{
// Takt Timer7 freigeben
    RCC->APB1ENR |= RCC_APB1ENR_TIM7EN;
    
   
// GPIOA configuration
// 2 MHz, Push-Pull und keine Pullup/-down-Widerstnde als default    
/* ist nicht mehr notwendig
    GPIOA->MODER |= (1 << (XL * 2) |                    // XL auf Ausgang
                     1 << (XR * 2) |                    // XR auf Ausgang
                     1 << (YD * 2) |                    // YD auf Ausgang
                     1 << (YU * 2));                    // YU auf Ausgang
*/
    
    RCC->APB1ENR |= RCC_APB1ENR_TIM7EN;
    TIM7->ARR = 1000-1;                                 // Teiler fr 1ms
    TIM7->PSC = SystemCoreClock/1000000/2-1;            // Vorteiler auf 1 MHz
    TIM7->CR1 = 0x1;                                    // starten
    TIM7->DIER = 0x1;                                   // und Interrupt freigeben
    
    NVIC_EnableIRQ(TIM7_IRQn);
    NVIC_SetPriority(TIM7_IRQn,PRIO_TIM7);              // geringe Prioritaet
    adc12_init();
}  


void lese_touch(int *t_x, int *t_y)
{
    TIM7->DIER &= ~TIM_DIER_UIE;    // Interrupt sperren
    __ISB();                        // Ausfuehrung abwarten
    *t_x = touch_x;
    *t_y = touch_y;
    TIM7->DIER |= TIM_DIER_UIE;     // und Interrupt freigeben
}



int get_tft_touch(int *x, int *y)   // fr direkte Verwendung
{
int temp = touch_status;
  if(temp) touch_status = 0;
  lese_touch(x,y);
  return(temp);	
}

    
void schreibe_piepser(uint8_t ein_aus)
{
  ein_aus = (ein_aus ^ 1) & 1;          // invertierte Ausgabe
  set_pin(GPIOC, PIEPSER, ein_aus);     // ein- oder ausschalten
  GPIOC->MODER |= 1 << (PIEPSER * 2);   // auf Ausgang
}    

void schreibe_blinkfrequenz(uint8_t wert)
{
	if(wert == 255) wert = 0;	// 255 oder 0: kein blinken
	t_blink = wert * 10;	    // auf 10 ms-Schritte bringen
}	

void set_timer(uint32_t wert)
{
	timer = wert;
}

uint32_t get_timer()
{
	return(timer);
}	

void sync()
{
	sync_flag = 0;
	while(!sync_flag);
}

void wait(int16_t zeit)
{
	while(zeit-- > 0) sync();
}

void tft_alarm(int16_t wert)
{
	schreibe_piepser(1);          // einschalten
	tft_alarm_count = wert;
}			
  
void blinken_ein()
{ blink_bit = 1;}

void blinken_aus()
{ blink_bit = 0;}


void TIM7_IRQHandler(void)
{
static uint16_t blink_ctr;
    TIM7->SR = 0;
    timer++;
    sync_flag = 1;
	if(tft_alarm_count) {
		tft_alarm_count--;
		if(tft_alarm_count == 0) 
			schreibe_piepser(0);	// abschalten
	}	

	if(t_blink) {	
		blink_ctr++;
		if(blink_ctr >= t_blink) {
			blink_ctr = 0;
			if(blink_bit) {
				blink_bit = 0;
			} else {
				blink_bit = 1;
			}
		}	
	} else {
        blink_bit = 0;	// kein Blinken
    }       
    dekodiere_touch();
    teste_touch_status();
}

